;; Routines to Call dpatch and use it from Emacs.
;; copyright 2003, 2005 Junichi Uekawa.

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; dpatch.el is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with your Debian installation, in /usr/share/common-licenses/GPL
;; If not, write to the Free Software Foundation, 675 Mass Ave,
;; Cambridge, MA 02139, USA.

(defgroup dpatch nil "dpatch mode"
  :group 'tools
  :prefix "dpatch-")



;;;###autoload(add-to-list 'auto-mode-alist '("debian/patches/.*\\.dpatch\\'" . dpatch-mode))
;;;###autoload(add-to-list 'auto-mode-alist '("debian/patches/00list\\'" . dpatch-00list-mode))
(add-to-list 'auto-mode-alist '("debian/patches/.*\\.dpatch\\'" . dpatch-mode))
(add-to-list 'auto-mode-alist '("debian/patches/00list\\'" . dpatch-00list-mode))

(defun dpatch-get-debian-dir ()
  "Find the directory with debian/ dir, and get the dir name.
You will find the debian/ dir from this directory.
"
  (let* ((looking-dir (expand-file-name (concat default-directory "."))))
    (while (not (file-accessible-directory-p (concat looking-dir "/debian")))
      (progn 
	(if (string= looking-dir "/")
	    (error "Cannot find debian dir anywhere"))
	(setq looking-dir (expand-file-name (expand-file-name (concat looking-dir "/.."))))))
    looking-dir))

;;; BEGIN dpatch-list-patch

;;;###autoload
(defun dpatch-list-patch ()
  "Obtain list of patches for this Debian package, using dpatch-list-patch command."
  (interactive)
  (let* ((my-default-directory (concat (dpatch-get-debian-dir) "/"))
	 (dpatch-buffer (concat "*dpatch*" my-default-directory))
	 (dpatch-process (concat "dpatch-process-" my-default-directory)))
    (switch-to-buffer dpatch-buffer)
    (dpatch-list-patch-mode)
    (setq default-directory my-default-directory)
    (kill-region (point-min) (point-max))
    (start-process dpatch-process dpatch-buffer "/usr/bin/dpatch-list-patch")))

(defun dpatch-list-patch-mode ()
  "Mode to view dpatch-list-patch output.
\\{dpatch-list-patch-mode-map}"
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'dpatch-list-patch-mode)
  (setq mode-name "dpatch-list-patch")
  (mapcar 'make-local-variable '(font-lock-defaults))
  (use-local-map dpatch-list-patch-mode-map)
  (set-syntax-table dpatch-list-patch-mode-syntax-table)
  (setq font-lock-defaults 
	'(
					;keywords start here
	  (("^  [^ ].*:$" . font-lock-string-face)
	   ("^[^ ].*:$" . font-lock-keyword-face)
	   ("^    [^ ].*$" . font-lock-comment-face)
	   )
	  nil		;keywords-only
	  nil		;case-fold
	  ()		;syntax-alist
	  ))
  (run-hooks 'dpatch-list-patch-mode-hook)
)

(defvar dpatch-list-patch-mode-map nil "Keymap for dpatch-list-patch mode.")
(defvar dpatch-list-patch-mode-syntax-table nil "Syntax table for dpatch-list-patch mode.")
(if dpatch-list-patch-mode-syntax-table
         ()              ; Do not change the table if it is already set up.
       (setq dpatch-list-patch-mode-syntax-table (make-syntax-table))
       (modify-syntax-entry ?\" ".   " dpatch-list-patch-mode-syntax-table)
       (modify-syntax-entry ?\\ ".   " dpatch-list-patch-mode-syntax-table)
       (modify-syntax-entry ?' "w   " dpatch-list-patch-mode-syntax-table))

(if dpatch-list-patch-mode-map
    nil
  (setq dpatch-list-patch-mode-map (make-sparse-keymap))
  (define-key dpatch-list-patch-mode-map "\C-m"
    'dpatch-list-patch-find-file))

(defun dpatch-list-patch-find-file ()
  "Find the dpatch file corresponding to the current cursor position"
  (interactive)
  (save-excursion
    (beginning-of-line)
    (if (looking-at "^\\(debian/patches/[^ ]+\\) \\([^:]+\\):$")
	(find-file (match-string 1))
      (error "Not on dpatch filename line"))))
;;; END dpatch-list-patch

;;; BEGIN dpatch-00list-mode
(defun dpatch-00list-mode ()
  "Mode to view 00list files found in dpatch source.
\\{dpatch-00list-mode-map}"
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'dpatch-00list-mode)
  (setq mode-name "dpatch-00list")
  (mapcar 'make-local-variable '(font-lock-defaults))
  (use-local-map dpatch-00list-mode-map)
  (set-syntax-table dpatch-00list-mode-syntax-table)
  (setq font-lock-defaults 
	'(
					;keywords start here
	  (("^[0-9]+_[^ ]+$" . font-lock-keyword-face)
	   ("^#.*$" . font-lock-comment-face)
	   )
	  nil		;keywords-only
	  nil		;case-fold
	  ()		;syntax-alist
	  ))
  (run-hooks 'dpatch-00list-mode-hook))

(defvar dpatch-00list-mode-map nil "Keymap for dpatch-00list mode.")
(defvar dpatch-00list-mode-syntax-table nil "Syntax table for dpatch-00list mode.")
(if dpatch-00list-mode-syntax-table
         ()              ; Do not change the table if it is already set up.
       (setq dpatch-00list-mode-syntax-table (make-syntax-table))
       (modify-syntax-entry ?\" ".   " dpatch-00list-mode-syntax-table)
       (modify-syntax-entry ?\\ ".   " dpatch-00list-mode-syntax-table)
       (modify-syntax-entry ?' "w   " dpatch-00list-mode-syntax-table))

(if dpatch-00list-mode-map
    nil
  (setq dpatch-00list-mode-map (make-sparse-keymap))
  (define-key dpatch-00list-mode-map "\C-m"
    'dpatch-00list-find-file)
  (define-key dpatch-00list-mode-map "+"
    'dpatch-00list-add-file))

(defun dpatch-00list-find-file ()
  "Find the dpatch file corresponding to the current cursor position"
  (interactive)
  (save-excursion
    (beginning-of-line)
    (if (looking-at "^\\([0-9][^\n ]+\\)$") ;this is a rough regexp
	(find-file (concat (match-string 1) ".dpatch"))
      (error "Not on a dpatch file line"))))

(defun dpatch-00list-add-file (dpatch-file-name)
  "Add a dpatch file to inclusion"
  (interactive "fdpatch file: ")
  (setq dpatch-file-name (expand-file-name dpatch-file-name)) ;canonicalize
  ;; find the 00list file
  (with-current-buffer
   (find-file-noselect
    (concat
     (dpatch-get-debian-dir) "/debian/patches/00list"))
   (next-line 1)
   (beginning-of-line)
   (if (string-match "debian/patches/\\([^ ]+\\)\\.dpatch$" dpatch-file-name)
       (insert
	(concat (match-string 1 dpatch-file-name) "\n"))       
     (error "Invalid dpatch filename"))))
;;; END dpatch-00list-mode

;;; BEGIN dpatch-mode

(require 'diff-mode)
;; shamelessly taken from diff-mode:
(defvar dpatch-font-lock-keywords
      (cons '("^[#]+ DP:.*$" . font-lock-keyword-face)
	    diff-font-lock-keywords)
      "Keywords used for font-lock in dpatch files")
(defvar dpatch-font-lock-defaults
      '(dpatch-font-lock-keywords t nil nil nil (font-lock-multiline . nil))
      "Font lock used for dpatch files")

(define-derived-mode dpatch-mode diff-mode "dpatch"
  "Mode to view dpatch files.
\\{dpatch-mode-map}"
  (set (make-local-variable 'font-lock-defaults) dpatch-font-lock-defaults)
  (run-hooks 'dpatch-mode-hook))

(defvar dpatch-mode-map nil "Keymap for dpatch mode.")
(defvar dpatch-mode-syntax-table nil "Syntax table for dpatch mode.")
(if dpatch-mode-syntax-table
         ()              ; Do not change the table if it is already set up.
       (setq dpatch-mode-syntax-table (make-syntax-table))
       (modify-syntax-entry ?\" ".   " dpatch-mode-syntax-table)
       (modify-syntax-entry ?\\ ".   " dpatch-mode-syntax-table)
       (modify-syntax-entry ?' "w   " dpatch-mode-syntax-table))
;;; END dpatch-mode

(provide 'dpatch)
